/*
 * Copyright 2017 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.kappich.pat.gnd.
 * 
 * de.kappich.pat.gnd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.kappich.pat.gnd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.kappich.pat.gnd.  If not, see <http://www.gnu.org/licenses/>.

 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436 
 * mail: <info@kappich.de>
 */

package de.kappich.pat.gnd.configBrowser;

import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.sys.funclib.kappich.annotations.NotNull;
import de.bsvrz.sys.funclib.kappich.properties.PropertyClass;
import de.bsvrz.sys.funclib.kappich.properties.PropertyName;

import java.util.*;

/**
 * <code>NetProperties</code> ist eine Klasse, die {@link ConfigObjectProperties}
 * um die Informationen für Objekte vom Typ <code>Netz</code> erweitert.
 *
 * @author Kappich Systemberatung
 * @version $Revision$
 */
@PropertyClass
public class NetProperties extends ConfigObjectProperties {
	private static final Object _configurationAccess = new Object();
	private static boolean _netToNetsMapInitialized = false;
	private static final Map<SystemObject, List<Object>> _netToNetsMap = new HashMap<>();

	/**
	 * Der Konstruktor.
	 *
	 * @param systemObject ein SystemObject
	 */
	public NetProperties(@NotNull final SystemObject systemObject) {
		super(systemObject);
	}

	/**
	 * Diese Methode bestimmt, welche Netz-Eigenschaften für äußere Straßensegmente
	 * angezeigt werden.
	 *
	 * @return ein ConfigObjectProperties-Objekt oder eine Liste davon
	 */
	@SuppressWarnings("SynchronizationOnStaticField")
	@PropertyName(name = "SuperNetze", sortKey = 10)
	public Object getNetProperties() {
		synchronized(_configurationAccess) {
			if(!_netToNetsMapInitialized) {
				initializeLookup();
			}
		}
		List<Object> list = _netToNetsMap.get(_systemObject);
		if(null == list) {
			return Collections.EMPTY_LIST;
		}
		else if(list.size() == 1) {
			return list.get(0);
		}
		else {
			return list;
		}
	}

	private void initializeLookup() {
		_netToNetsMap.clear();

		DataModel configuration = _systemObject.getDataModel();
		SystemObjectType netType = configuration.getType("typ.netz");
		List<SystemObject> superNets = netType.getElements();
		for(SystemObject superNet : superNets) {
			ConfigurationObject configurationObject = (ConfigurationObject) superNet;
			ObjectSet objectSet = configurationObject.getObjectSet("NetzBestandTeile");
			List<SystemObject> elements = objectSet.getElements();
			for(SystemObject element : elements) {
				if(_netToNetsMap.containsKey(element)) {
					_netToNetsMap.get(element).add(createConfigObjectProperties(superNet));
				}
				else {
					List<Object> newList = new ArrayList<>(1);
					newList.add(createConfigObjectProperties(superNet));
					_netToNetsMap.put(element, newList);
				}
			}
		}
		_netToNetsMapInitialized = true;
	}
}
